package com.xiaoniu.niucai.config;

import com.xiaoniu.niucai.entity.SysCrontab;
import com.xiaoniu.niucai.service.ISysCrontabService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;

import java.util.List;

@Slf4j
@Configuration
public class ScheduledConfig implements SchedulingConfigurer {
    @Autowired
    private ApplicationContext context;

    @Autowired
    JobCronConfig jobCronConfig;

    @Autowired
    ISysCrontabService iSysCrontabService;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        ThreadPoolTaskScheduler taskScheduler = new ThreadPoolTaskScheduler();
        // 10 个线程
        taskScheduler.setPoolSize(jobCronConfig.getPoolSize());
        taskScheduler.initialize();
        scheduledTaskRegistrar.setTaskScheduler(taskScheduler);

        List<SysCrontab> jobList = iSysCrontabService.list();
        for (SysCrontab job : jobList) {
            try {
                Runnable task = (Runnable) context.getBean(Class.forName(job.getClassName()));
                scheduledTaskRegistrar.addCronTask(task, job.getCronSetting().trim());

                log.info("任务【{}】: 启动成功", job.getTitle());
            } catch (ClassNotFoundException e) {
                log.error("{} 任务类不存在", job.getClassName());
                e.printStackTrace();
            }
        }
    }
}
